import { createToken, createPlugin, unescape } from 'fusion-core';
import { FetchToken } from 'fusion-tokens';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
const I18nLoaderToken = createToken('I18nLoaderToken');

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var createLoader = null;

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env node */

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/* eslint-env browser */
function loadTranslations() {
  const element = document.getElementById('__TRANSLATIONS__');

  if (!element) {
    throw new Error('[fusion-plugin-i18n] - Could not find a __TRANSLATIONS__ element');
  }

  try {
    return JSON.parse(unescape(element.textContent));
  } catch (e) {
    throw new Error('[fusion-plugin-i18n] - Error parsing __TRANSLATIONS__ element content');
  }
}

const HydrationStateToken = createToken('HydrationStateToken');

const pluginFactory$1 = () => createPlugin({
  deps: {
    fetch: FetchToken.optional,
    hydrationState: HydrationStateToken.optional
  },
  provides: ({
    fetch = window.fetch,
    hydrationState
  } = {}) => {
    class I18n {
      constructor() {
        const {
          chunks,
          localeCode,
          translations
        } = hydrationState || loadTranslations();
        this.loadedChunks = chunks || [];
        this.localeCode = localeCode;
        this.translationMap = translations || {};
      }

      load(chunkIds) {
        return new Promise(function ($return, $error) {
          const unloaded = chunkIds.filter(id => this.loadedChunks.indexOf(id) < 0);
          const fetchOpts = {
            method: 'POST',
            headers: _objectSpread({
              Accept: '*/*'
            }, this.localeCode ? {
              'X-Fusion-Locale-Code': this.localeCode
            } : {})
          };

          if (unloaded.length > 0) {
            // Don't try to load translations again if a request is already in
            // flight. This means that we need to add unloaded chunks to
            // loadedChunks optimistically and remove them if some error happens
            this.loadedChunks = [...this.loadedChunks, ...unloaded];
            const ids = unloaded.join(','); // TODO(#3) don't append prefix if injected fetch also injects prefix

            return $return(fetch(`/_translations?ids=${ids}`, fetchOpts).then(r => r.json()).then(data => {
              for (const key in data) this.translationMap[key] = data[key];
            }).catch(err => {
              // An error occurred, so remove the chunks we were trying to load
              // from loadedChunks. This allows us to try to load those chunk
              // translations again
              this.loadedChunks = this.loadedChunks.filter(chunk => unloaded.indexOf(chunk) === -1);
              throw err;
            }));
          }

          return $return();
        }.bind(this));
      }

      translate(key, interpolations = {}) {
        const template = this.translationMap[key];
        return template ? template.replace(/\${(.*?)}/g, (_, k) => interpolations[k]) : key;
      }

    }

    const i18n = new I18n();
    return {
      from: () => i18n
    };
  }
});

var clientPlugin = true && pluginFactory$1();

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
const I18nToken = createToken('I18nToken');
const plugin = clientPlugin;

export default plugin;
export { I18nToken, I18nLoaderToken, HydrationStateToken, createLoader as createI18nLoader };
//# sourceMappingURL=browser.es2015.es.js.map
